local _Util_Math = Util.Math
local UI_TYPE		= CppEnums.PA_UI_CONTROL_TYPE

local DeliveryIconTemplate = UI.getChildControl( Panel_Win_Worldmap, "Static_Icon_Wagon" );
_PA_ASSERT(DeliveryIconTemplate ~= nil, "비정상입니다!");

DeliveryIconTemplate:SetShow(false)

local templateIconSize = {
	width	= DeliveryIconTemplate:GetSizeX(),
	height	= DeliveryIconTemplate:GetSizeY()
};

local DELIVERY_ICON_ID = 'DeliveryIcon'

-- 아이콘들 Pool
local iconPool = ControlPool.createPool( Panel_Win_Worldmap )
iconPool:addCategory( DELIVERY_ICON_ID, UI_TYPE.PA_UI_CONTROL_STATIC, DeliveryIconTemplate, 'Static_DeliveryIcon_' )

-- 마차들 아이콘. 운송중일때만 표시된다.
local DeliveryActors = {}

-- 운송 가능한 노드들 아이콘. 운송 대기 또는 운송 완료 아이템이 존재할 때만 표시된다.
local DeliveryNodes = {}

-- 마차에 운송 아이템이 추가/삭제 될때 호출된다! (매 프레임 호풀되지 않는다!)
function WorldMapWindow_DelivererActor_AddRemove( actorKeyRaw, isHave )
	local deliveryIcon = DeliveryActors[actorKeyRaw]

	if isHave then
		-- Icon 이 존재하지 않는다면, Pool 에서 받아와서 출력한다!
		if nil == deliveryIcon then
			deliveryIcon = iconPool:getOrCreateControl( DELIVERY_ICON_ID )
			deliveryIcon:SetShow(true)
			DeliveryActors[actorKeyRaw] = deliveryIcon
		end
	else
		-- Icon 이 존재한다면, Pool 에 반납한다!
		if nil ~= deliveryIcon then
			iconPool:returnToPool( DELIVERY_ICON_ID, deliveryIcon )
			DeliveryActors[actorKeyRaw] = nil
		end
	end
end

function changeDeliveryIcon( deliveryIcon, deliveryType )
	if 0 ~= deliveryType then
		if 1 == deliveryType then
			deliveryIcon:ChangeTextureInfoName( "New_UI_Common_forLua/Widget/WorldMap/WorldMap_Etc_00.dds" )
			local x1, y1, x2, y2 = setTextureUV_Func( deliveryIcon, 43, 127, 84, 160 )
			deliveryIcon:getBaseTexture():setUV(  x1, y1, x2, y2  )
			deliveryIcon:setRenderTexture(deliveryIcon:getBaseTexture())
		elseif 2 == deliveryType and isSelfPcDelivery() then		-- pc운송
			deliveryIcon:ChangeTextureInfoName( "New_UI_Common_forLua/Widget/WorldMap/WorldMap_Etc_00.dds" )
			local x1, y1, x2, y2 = setTextureUV_Func( deliveryIcon, 1, 127, 42, 160 )
			deliveryIcon:getBaseTexture():setUV(  x1, y1, x2, y2  )
			deliveryIcon:setRenderTexture(deliveryIcon:getBaseTexture())			
		elseif 3 == deliveryType then
			deliveryIcon:ChangeTextureInfoName( "New_UI_Common_forLua/Widget/WorldMap/WorldMap_Etc_00.dds" )
			local x1, y1, x2, y2 = setTextureUV_Func( deliveryIcon, 85, 127, 127, 160 )
			deliveryIcon:getBaseTexture():setUV(  x1, y1, x2, y2  )
			deliveryIcon:setRenderTexture(deliveryIcon:getBaseTexture())
		end
	end
end

-- 매 프레임 호출된다. 운송중인 마차의 위치로 icon 위치를 세팅해준다.
function WorldMapWindow_DelivererActor_PositionUpdate(deliveryNo, actorKeyRaw)
	if not Panel_Win_Worldmap:GetShow() then
		return;
	end

	-- icon 이 존재하면, 운송중인 아이템이 있다는 의미이다.
	-- 아이콘이 존재하지 않으면, 운송중인 아이템이 없다는 의미이다.
	local deliveryIcon = DeliveryActors[actorKeyRaw]
	local deliveryType = delivery_infoWrapper( deliveryNo )
	if nil == deliveryIcon then
		if 0 ~= deliveryType then
			WorldMapWindow_DelivererActor_AddRemove( actorKeyRaw, true )
			deliveryIcon = DeliveryActors[actorKeyRaw]
		end
		--return;
	end
	
	-- 다시 검사가 필요할까..
	if nil == deliveryIcon then
		return
	end

	--deliveryType = delivery_infoWrapper( deliveryNo )
	changeDeliveryIcon( deliveryIcon, deliveryType )

	if 2 == deliveryType and false == isSelfPcDelivery() then
		deliveryIcon:SetShow( false )
		return
	end

	local actorProxyWrapper = getActor(actorKeyRaw)
	if	( nil == actorProxyWrapper)	then
		return
	end
	
	local deliveryProxy		= actorProxyWrapper:get()
	if	( nil == deliveryProxy)	then
		return
	end
	
	local deliveryPosition	= float3(deliveryProxy:getPositionX(), deliveryProxy:getPositionY(), deliveryProxy:getPositionZ());

	local pos2D = worldmapScenetoScreenPosition(deliveryPosition.x, deliveryPosition.y + 5000, deliveryPosition.z);
	if (pos2D.x < 0) or (pos2D.y < 0) or (1 < pos2D.x) or (1 < pos2D.y) then
		deliveryIcon:SetShow( false )
		return
	end
	pos2D.x 		= pos2D.x * WorldMapWindow.world3DMapStatic:GetSizeX();
	pos2D.y 		= pos2D.y * WorldMapWindow.world3DMapStatic:GetSizeY();

	local iconScaler		= 1.5;
	local nodeSize = _Util_Math.calculateDepthScaler(deliveryPosition, templateIconSize);
	nodeSize.width	= nodeSize.width	* iconScaler;
	nodeSize.height	= nodeSize.height	* iconScaler;

	deliveryIcon:SetDepth(WorldMapWindow_GetDepthFromPos3D(getWorldMapCameraPos(), deliveryPosition));
	deliveryIcon:SetSize(nodeSize.width, nodeSize.height);
	deliveryIcon:SetPosX(pos2D.x - nodeSize.width / 2);
	deliveryIcon:SetPosY(pos2D.y - nodeSize.height);
	deliveryIcon:SetShow( true )
end

-- 노드에 운송 대기 아이템이 추가/삭제 될때 호출된다.
function WorldMapWindow_DeliveryNode_AddRemove_StandbyItem( nodeKey, isHave )
	local nodeIcons = DeliveryNodes[nodeKey]
	if nil == nodeIcons then
		nodeIcons = {}
		DeliveryNodes[nodeKey] = nodeIcons
	end

	if isHave then
		-- Icon 이 존재하지 않는다면, Pool 에서 받아와서 출력한다!
		if nil == nodeIcons._sendIcon then
			nodeIcons._sendIcon = iconPool:getOrCreateControl( DELIVERY_ICON_ID )
		end
	else
		-- Icon 이 존재한다면, Pool 에 반납한다!
		if nil ~= nodeIcons._sendIcon then
			iconPool:returnToPool( DELIVERY_ICON_ID, nodeIcons._sendIcon )
			nodeIcons._sendIcon = nil
		end
	end
end

-- 노드에 운송 완료 아이템이 추가/삭제 될때 호출된다.
function WorldMapWindow_DeliveryNode_AddRemove_CompleteItem( nodeKey, isHave )
	local nodeIcons = DeliveryNodes[nodeKey]
	if nil == nodeIcons then
		nodeIcons = {}
		DeliveryNodes[nodeKey] = nodeIcons
	end

	if isHave then
		-- Icon 이 존재하지 않는다면, Pool 에서 받아와서 출력한다!
		if nil == nodeIcons._receiveIcon then
			nodeIcons._receiveIcon = iconPool:getOrCreateControl( DELIVERY_ICON_ID )
		end
	else
		-- Icon 이 존재한다면, Pool 에 반납한다!
		if nil ~= nodeIcons._receiveIcon then
			iconPool:returnToPool( DELIVERY_ICON_ID, nodeIcons._receiveIcon )
			nodeIcons._receiveIcon = nil
		end
	end
end

-- 매 프레임 호출된다. 운송대기, 운송완료 아이템이 있는 노드 위치로 icon 위치를 세팅해준다!
local WorldMapWindow_DeliveryNode_UpdatePos = function( nodeKey )
	local	nodeIcons	= DeliveryNodes[nodeKey]
	if nil == nodeIcons then
		return
	end

	local	sendIcon	=	nodeIcons._sendIcon
	local	reveiveIcon	=	nodeIcons._receiveIcon
	if (nil == sendIcon) and (nil == reveiveIcon) then
		return
	end

	
	
	if ( nil == WorldMapWindow.nodeDataList[nodeKey] ) then
		return
	end
	
	local nodePosition		= WorldMapWindow.nodeDataList[nodeKey]:getStaticStatus():getPosition();
	local nodePosition2D	= worldmapScenetoScreenPosition(nodePosition.x, nodePosition.y, nodePosition.z);
	if (nodePosition2D.x < 0) or (nodePosition2D.y < 0) or (1 < nodePosition2D.x) or (1 < nodePosition2D.y) then
		if nil ~= sendIcon then
			sendIcon:SetShow( false )
		end
		if nil ~= reveiveIcon then
			reveiveIcon:SetShow( false )
		end
	end

	local cameraPos = getWorldMapCameraPos();
	local depth = WorldMapWindow_GetDepthFromPos3D(cameraPos, nodePosition)

	local nodeScaler = 1.5;
	local nodeSize	= _Util_Math.calculateDepthScaler(nodePosition, templateIconSize);
	nodeSize.width	= nodeSize.width	* nodeScaler;
	nodeSize.height	= nodeSize.height	* nodeScaler;

	nodePosition2D.x 		= nodePosition2D.x * WorldMapWindow.world3DMapStatic:GetSizeX() - nodeSize.width / 2;
	nodePosition2D.y 		= nodePosition2D.y * WorldMapWindow.world3DMapStatic:GetSizeY() - nodeSize.height / 2;

	if nil ~= sendIcon then
		sendIcon:SetDepth(depth);
		sendIcon:SetSize(nodeSize.width, nodeSize.height);
		sendIcon:SetPosX(nodePosition2D.x);
		sendIcon:SetPosY(nodePosition2D.y);
		sendIcon:SetShow( true )
	end
	if nil ~= reveiveIcon then
		reveiveIcon:SetDepth(depth);
		reveiveIcon:SetSize(nodeSize.width, nodeSize.height);
		reveiveIcon:SetPosX(nodePosition2D.x);
		reveiveIcon:SetPosY(nodePosition2D.y);
		reveiveIcon:SetShow( true )
	end
end

function WorldMapWindow_DeliveryNode_PositionUpdate()
	if not Panel_Win_Worldmap:GetShow() then
		return;
	end

	for nodeKey, nodeButton in pairs(DeliveryNodes) do
		WorldMapWindow_DeliveryNode_UpdatePos( nodeKey )
	end
end

-- 모든 아이콘을 풀에 반납!
local WorldMapWindow_OnHide = function()
	for _,icon in pairs(DeliveryActors) do
		iconPool:returnToPool( DELIVERY_ICON_ID, icon )
	end
	DeliveryActors = {}

	for _,nodeIcons in pairs(DeliveryNodes) do
		if nil ~= nodeIcons._sendIcon then
			iconPool:returnToPool( DELIVERY_ICON_ID, nodeIcons._sendIcon )
		end
		if nil ~= nodeIcons._receiveIcon then
			iconPool:returnToPool( DELIVERY_ICON_ID, nodeIcons._receiveIcon )
		end
	end
	DeliveryNodes = {}
end
WorldMapWindow_Set_OnShowHide_Handler( false, WorldMapWindow_OnHide )

