if nil == WorldMapWindow then
	WorldMapWindow = {};
end

-- WorldMap_UpdatePerFrame 에는 매프레임 애니메이션 할 녀석들만,
-- 위치가 고정이고 카메라가 변경될때만 갱신이 필요한 녀석은 WorldMapWindow_RefreshOnUpdateCamera 함수에 등록한다.
function WorldMapWindow_RefreshOnUpdateCamera()
	WorldMapWindow_OnViewModeUpdate();		-- Check
	--WorldMapWindow.HouseholdIcon_InvalidatePosition = true		-- 고정집 아이콘 위치 업데이트
	--WorldMapWindow.NodeButtons_InvalidatePosition = true		-- Node 버튼 위치 업데이트
	WorldMapWindow_Update_TerritoryNameAndExplorationPoint()
end

WorldMapWindow.callerList = {}


WorldMapWindow.ShowRunList = {
	[WorldMapWindow.RenderType.WMS_EXPLORE_PLANT] = function()
		local functor = WorldMapWindow.callerList.MapWindowEvent_CameraPosChange_Functor[WorldMapWindow.RenderType.WMS_EXPLORE_PLANT]
		if ( nil ~= functor ) then
			functor()
		end
		WorldMapWindow_Init_MentalCardInformationUI()
	end,

	[WorldMapWindow.RenderType.WMS_REGION] = function()
		ToClient_worldmap_resetSelectedRegionInfo();
	end,
}


------------------------------------------------------------------------------------------
-- WorldMap_UpdatePerFrame
------------------------------------------------------------------------------------------
WorldMapWindow.callerList.MapWindowEvent_UpdatePerFrame_Functor = 
{
	[WorldMapWindow.RenderType.WMS_GLOBAL] = function()
		local targetUI = WorldMapWindow.uiList[WorldMapWindow.RenderType.WMS_GLOBAL]
		if ( targetUI:GetShow() ) then
			WorldMapWindow_TerritoryNameAndExplorationPoint_PositionUpdate()

			targetUI:SetDepthSort()
		end
	end,
	[WorldMapWindow.RenderType.WMS_EXPLORE_PLANT] = function()
		local targetUI = WorldMapWindow.uiList[WorldMapWindow.RenderType.WMS_EXPLORE_PLANT]
		if ( targetUI:GetShow() ) then
			WorldMapWindow_Update_NodeAlphaChangeAnimation()
			WorldMapWindow_HouseholdIcon_PositionUpdate()
			WorldMapWindow_QuestAnimation()
			WorldMapWindow_Refresh_CircleLine()
			WorldMapWindow_Refresh_SunLine()
			WorldMapWindow_Refresh_SunIcon()
			WorldMapWindow_Refresh_NEWS_Position()
			---------------------------------
			-- 제일 많이 먹음
			WorldMapWindow_Refresh_CircleLine_VillageNameList()	-- dev : 1.5~2ms
			WorldMapWindow_RefreshPosition_NodeButtonList()		-- dev : 8~9ms
			WorldMapWindow_Refresh_MentalCardInformationUI()	-- release : 1.0ms
			---------------------------------
			
			WorldMapWindow_RefreshPosition_NodeMenu()
			WorldMapWindow_DeliveryNode_PositionUpdate()
			WorldMapWindow_SpecialActorIcon_PositionUpdate()
			WorldMapWindow_ActorIcon_PositionUpdate()
			WorldMap_NodeIcon_WorkProgress_Update()
			WorldMapWindow_PositionGuidePin_PositionUpdate()
			FGlobal_WorldMap_IconName_UpdatePosition()
			
			targetUI:SetDepthSort()
		end
	end,
	[WorldMapWindow.RenderType.WMS_REGION] = function()
		local targetUI = WorldMapWindow.uiList[WorldMapWindow.RenderType.WMS_REGION]
		if ( targetUI:GetShow() ) then
			WorldMapWindow_RefreshPosition_RegionButtonList();
			WorldMapWindow_RefreshPosition_NodeMenu()
			WorldMapWindow_UpdateDepthSort()
			WorldMapWindow_Refresh_GrapePercents()

			targetUI:SetDepthSort()
		end
	end,
}
function WorldMapWindowEvent_UpdatePerFrame()
	if ( Panel_Win_Worldmap:IsShow() == false ) then
		return
	end
	local renderType = ToClient_worldmap_getRenderState()
	local functor = WorldMapWindow.callerList.MapWindowEvent_UpdatePerFrame_Functor[renderType]
	if ( nil ~= functor ) and ( WorldMapWindow.RenderType.WMS_GLOBAL ~= renderType ) then
		functor()
	end
	local globalFunctor = WorldMapWindow.callerList.MapWindowEvent_UpdatePerFrame_Functor[ WorldMapWindow.RenderType.WMS_GLOBAL ]
	if ( nil ~= globalFunctor ) then
		globalFunctor()
	end
end
registerEvent("WorldMap_UpdatePerFrame",			"WorldMapWindowEvent_UpdatePerFrame");



------------------------------------------------------------------------------------------
-- WorldMap_CameraPosChange
------------------------------------------------------------------------------------------
WorldMapWindow.callerList.MapWindowEvent_CameraPosChange_Functor = 
{
	[WorldMapWindow.RenderType.WMS_EXPLORE_PLANT] = function()
		WorldMapWindow_RefreshOnUpdateCamera()
		WorldMapWindow_QuestPosition()
	end,
}
function WorldMapWindowEvent_CameraPosChange()
	local functor = WorldMapWindow.callerList.MapWindowEvent_CameraPosChange_Functor[ToClient_worldmap_getRenderState()]
	if ( nil ~= functor ) then
		functor()
	end
end
registerEvent("WorldMap_CameraPosChange",			"WorldMapWindowEvent_CameraPosChange");


------------------------------------------------------------------------------------------
-- WorldMap_NewNode
------------------------------------------------------------------------------------------
function WorldMapWindowEvent_NewNode(node)
	WorldMapWindow_NewNode(node)
	WorldMapWindow_NodeButton_New(node)
	WorldMap_ChildNode_List_Update (node)
	WorldMapWindow_Insert_CircleLine_VillageName(node)
	WorldMapWindow_Refresh_CircleLine_VillageName(node)
end
registerEvent("WorldMap_NewNode",					"WorldMapWindowEvent_NewNode");


function WorldMapWindowEvent_NewRegion( regionGroupInfo )
	WorldMapWindow_NewRegion( regionGroupInfo );
	WorldMapWindow_RegionButton_New( regionGroupInfo );
end

registerEvent("WorldMap_NewRegion",					"WorldMapWindowEvent_NewRegion");


------------------------------------------------------------------------------------------
-- WorldMap_DeleteNode
------------------------------------------------------------------------------------------
function WorldMapWindowEvent_DeleteNode(nodeKey)
	WorldMapWindow_NodeButton_Delete(nodeKey)
	WorldMapWindow_DeleteNode(nodeKey)
end
registerEvent("WorldMap_DeleteNode",				"WorldMapWindowEvent_DeleteNode");



-- 순서가 중요하다. WorldMapWindow_Refresh_CircleLine() 가 가장 먼저 실행되어야 한다!
registerEvent("WorldMap_Invalid_ExploreData",		"WorldMapWindow_Invalid_TerritoryNameAndExplorationPoint")
registerEvent("EventWorldMapStateChange",			"WorldMapWindow_ChangeState")		-- Check
registerEvent("WorldMap_NodeUpgrade",				"WorldMapWindow_NotifyNodeUpgrade")
registerEvent("WorldMap_NodeWithdraw",				"WorldMapWindow_NotifyNodeUpgrade")
registerEvent("onScreenResize",						"WorldMapWindow_OnScreenResize");
registerEvent("WorldMap_ClearNode",					"WorldMapWindow_ClearNode")
registerEvent("WorldMap_Show",						"WorldMapWindow_Show")
registerEvent("WorldMap_Close",						"WorldMapWindow_OnClose")
registerEvent("WorldMap_ElementShow",				"WorldMapWindow_ElementShow")
registerEvent("EventQuestListChanged",				"WorldMapWindow_QuestListChanged")
registerEvent("WorldMap_WorkerUpdate",				"WorldMapWindow_MyWorkerUpdate")
registerEvent("WorldMap_StopWorkerWorking",			"WorldMapWindow_StopMyWorkerWorking")
registerEvent("EventActorDelete", 					"WorldMapWindow_ActorIcon_Delete")
registerEvent("updateGuildInfo",					"WorldMapWindow_GuildInfoChange")
registerEvent("EventDelivererUpdate",				"WorldMapWindow_DelivererActor_AddRemove")
registerEvent("WorldMap_DeliveryUpdate",			"WorldMapWindow_DelivererActor_PositionUpdate")
registerEvent("EventRegionDeliverySendUpdate",		"WorldMapWindow_DeliveryNode_AddRemove_StandbyItem")
registerEvent("EventRegionDeliveryReceiveUpdate",	"WorldMapWindow_DeliveryNode_AddRemove_CompleteItem")
registerEvent("WorldMap_WorkerDataUpdate",			"WorldMapWindow_WaitWorkerListUpdate")
registerEvent("FromClient_DeliveryRequestAck",		"DeliveryRequest_UpdateRequestSlotData");
registerEvent("EventDeliveryInfoUpdate",			"DeliveryInformation_UpdateSlotData")
registerEvent("EventWarehouseUpdate",				"ToClient_WorldMapWindow_WareHouseUpdate")
WorldMapWindow:SetFullScreen(true);

